//
// Title:   AffineRegistration.h
// Created: March 1, 2003
// Authors: Daniel Cotting, Oliver Knoll, Mark Pauly, Matthias Zwicker
//
// Copyright (c) 2001, 2002, Computer Graphics Lab, ETH Zurich
//
// This file is part of the Pointshop3D system.
// See http://www.pointshop3d.com/ for more information.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later
// version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General
// Public License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place, Suite 330,
// Boston, MA 02111-1307 USA
// 
// Contact info@pointshop3d.com if any conditions of this
// licensing are not clear to you.
//

#ifndef __AFFINEREGISTRATION_H_
#define __AFFINEREGISTRATION_H_

#include <qintdict.h>
#include <qarray.h>

#include "../../../../Core/DataStructures/src/SurfelCollection.h"
#include "../../../../Core/MarkerManager/src/MarkerInterface.h"

/**
 * This class implements a tool with which the user can register two objects
 * using an affine transformation (manually defined by specifying corresponding
 * points on both objects.
 */
class AffineRegistration : public QObject {

	Q_OBJECT

public:

	AffineRegistration();
	virtual ~AffineRegistration();

	void registerObjects(int firstObjectID, int currentObjectID, QList<MarkerInterface>* originalObjectMarkers, QList<MarkerInterface>* verificationObjectMarkers);	

private:
	
private slots:

	// connected to SelectionTool: selectionChanged - release the data structure
	void handleSelectionChanged (SurfelInterface::Flag selectionFlag);

};

#endif  // __AFFINEREGISTRATION_H_

// Some Emacs-Hints -- please don't remove:
//
//  Local Variables:
//  mode:C++
//  tab-width:4
//  End:
