//
// Title:   AffineRegistrationTool.h
// Created: March 1, 2003
// Authors: Daniel Cotting, Oliver Knoll, Mark Pauly, Matthias Zwicker
//
// Copyright (c) 2001, 2002, Computer Graphics Lab, ETH Zurich
//
// This file is part of the Pointshop3D system.
// See http://www.pointshop3d.com/ for more information.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later
// version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General
// Public License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place, Suite 330,
// Boston, MA 02111-1307 USA
// 
// Contact info@pointshop3d.com if any conditions of this
// licensing are not clear to you.
//

#ifndef __AFFINEREGISTRATIONTOOL_H_
#define __AFFINEREGISTRATIONTOOL_H_

#include <qwidget.h>
#include <qtoolbutton.h>
#include <qpoint.h>
#include <qcursor.h>
#include <qobject.h>
#include <qstring.h>
#include <qlist.h>
#include <qintdict.h>

#include "../../../../Core/Interfaces/src/RendererInterface.h"
#include "../../../../Core/Interfaces/src/ToolInterface.h"
#include "../../../../Core/Transformer/src/Transformer.h"
#include "../../../../Core/MarkerManager/src/MarkerManager.h"
#include "../../../../Core/MarkerManager/src/Marker2D.h"
#include "../../../../Core/MarkerManager/src/Marker3D.h"
#include "../UI/AffineRegistrationConfigurationWidget.h"
#include "AffineRegistration.h"
#include "AffineRegistrationDialogImpl.h"

/**
 * This class implements a tool with which two objects
 * can be registered by performing an affine transformation
 * that is manually defined by setting corresponding points
 * on both objects.
 */
class AffineRegistrationTool : public ToolInterface {

	Q_OBJECT

public:

	TOOL_API AffineRegistrationTool();
	TOOL_API virtual ~AffineRegistrationTool();

	// ***********************
	// interface ToolInterface
	// ***********************

	TOOL_API QToolButton *addButtonToToolBar (QToolBar *toolBar);
	TOOL_API QToolButton *getToolButton();
	TOOL_API QCursor *getToolCursor() const;
	TOOL_API QWidget *getToolConfigurationWidget() const;
	TOOL_API bool isAvailable() const;

	TOOL_API void handleMousePressEvent   (QMouseEvent *me);
	TOOL_API void handleMouseReleaseEvent (QMouseEvent *me);
	TOOL_API void handleMouseMoveEvent    (QMouseEvent *me);
	TOOL_API void handleKeyPressEvent     (QKeyEvent   *ke);
	TOOL_API void handleKeyReleaseEvent   (QKeyEvent   *ke);

	TOOL_API void selectTool();
	TOOL_API void deselectTool();

	void removeSelectedMarkers();	
	void removeAllMarkers();	
	void toggleMarkers();	
	void registerObjects();	

public slots:

	void handleRendererWidgetRepainted();

private:

	// direct access for...
	friend class AffineRegistrationDialogImpl;

	// the last position of the mouse before the
	// current mouse drag event
	QPoint                    lastMouseDragPosition;
	QCursor                   *toolCursor;
	QToolButton               *affineRegistrationButton;

	// the configuration widget
	AffineRegistrationConfigurationWidget *configurationWidget;

	AffineRegistration        *affineRegistration;

	QList<MarkerInterface>	  *markers1;
	QList<MarkerInterface>	  *markers2;
	int						  firstObjectID;
	Marker3D                  *selectedMarker;          // the 3D marker which was clicked

	bool                      isDragging,               // true, if we are dragging a marker with the mouse
	                          keepMarker;               // false, if the marker is dragged over area where
	                                                    // no splats are available
	        
    QPoint                    affineRegistrationDialogPosition;
    QSize                     affineRegistrationDialogSize;

	bool checkToolAvailability() const;

};

// ****************
// access functions
// ****************

// extern "C" is needed so those functions can be accessed
// with GetProcAddress() (WIN32) and dlsym() (UNIX) respective
extern "C" {
	TOOL_API ToolInterface *createTool();
	TOOL_API const char *getToolName();
}

#endif  // __AFFINEREGISTRATIONTOOL_H_

// Some Emacs-Hints -- please don't remove:
//
//  Local Variables:
//  mode:C++
//  tab-width:4
//  End:
