//
// Title:   PCARegistration.h
// Created: November 28, 2002
// Authors: Daniel Cotting
//
// Copyright (c) 2001, 2002, Computer Graphics Lab, ETH Zurich
//
// This file is part of the Pointshop3D system.
// See http://www.pointshop3d.com/ for more information.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free
// Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.
//
// Contact info@pointshop3d.com if any conditions of this
// licensing are not clear to you.
//

#ifndef __PCAREGISTRATION_H_
#define __PCAREGISTRATION_H_

#include <qobject.h>

#include "../../../Core/Interfaces/src/PluginInterface.h"
#include "../../../Core/Interfaces/src/SurfelInterface.h"
#include "MySurfelList.h"
#include "RegistrationInterface.h"
#include "WatermarkInterface.h"


/**
 * Registers two surfel lists, that is both geometries are moved and rescaled to 
 * become as close as possible. In the ideal case both geometries will look more 
 * or less the same and will be located at the same position is space with the same
 * orientation. This is used as a preprocessing step before extracting the 
 * watermark in watermarking schemes using two objects (the original one and the
 * verification object) to get the watermark.
 * This implementation uses Principal Component Analysis (PCA) to perform that task.
 */
class PCARegistration : public RegistrationInterface {

public:
	
	/**
	 * Constructor of the <tt>PCARegistration</tt> class.
	 */
	PCARegistration();

	/**
	 * Destructor of the <tt>PCARegistration</tt> class.
	 */
	~PCARegistration();

	/**
	 * Registers the two geometries <tt>originalSurfelList</tt> and <tt>verificationSurfelList</tt>
	 * using Principal Component Analysis.
	 *
	 * @param originalSurfelList The list containing the surfels of the original object.
	 * @param verificationSurfelList The list containing the surfels of the verification object.
	 */
	void registerObjects(
		MySurfelList* originalSurfelList,
		MySurfelList* verificationSurfelList
	);

	/**
	 * Returns the name of this module.
	 *
	 * @return The name of this module.
	 */
	const char* getModuleName() const;

	/**
	 * Returns the description of this module.
	 *
	 * @return The description of this module.
	 */
	const char* getModuleDescription() const;
};

#endif  //__PCAREGISTRATION_H_

// Some Emacs-Hints -- please don't remove:
//
//  Local Variables:
//  mode:C++
//  tab-width:4
//  End:
