//
// Title:   Random.h
// Created: November 28, 2002
// Authors: Daniel Cotting
//
// Copyright (c) 2001, 2002, Computer Graphics Lab, ETH Zurich
//
// This file is part of the Pointshop3D system.
// See http://www.pointshop3d.com/ for more information.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free
// Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.
//
// Contact info@pointshop3d.com if any conditions of this
// licensing are not clear to you.
//

#ifndef RANDOM_H
#define RANDOM_H

/**
 * Random number generator based on Knuth and code from www.nr.com.
 *
 * Currently uniform random numbers and gaussian random numbers are implemented.
 *
 * @author Daniel Cotting
 */
class Random {

public:
	Random(long seed);		///< Constructor for the random number generator class.
    double getUniform();	///< Calculates a uniformly distributed pseudo random number.
	double getGaussian();   ///< Calculates a normally distributed pseudo random number.
private:
	int inext,inextp;
	long ma[56]; 			// The value 56 (range ma[1..55]) is special and
							// should not be modifed; see Knuth. 
};
#endif

// Some Emacs-Hints -- please don't remove:
//
//  Local Variables:
//  mode:C++
//  tab-width:4
//  End:
